@@ -103,11 +103,16 @@ class KeyBlobStorage:
 def key_creator(repository, args):
     for key in AVAILABLE_KEY_TYPES:
         if key.ARG_NAME == args.encryption:
+            assert key.ARG_NAME is not None
             return key.create(repository, args)
     else:
         raise ValueError('Invalid encryption mode "%s"' % args.encryption)
 
 
+def key_argument_names():
+    return [key.ARG_NAME for key in AVAILABLE_KEY_TYPES if key.ARG_NAME]
+
+
 def identify_key(manifest_data):
     key_type = manifest_data[0]
     if key_type == PassphraseKey.TYPE:
@@ -690,7 +695,7 @@ class KeyfileKey(ID_HMAC_SHA_256, KeyfileKeyBase):
         id = self.repository.id
         keyfile = os.environ.get('BORG_KEY_FILE')
         if keyfile:
-            return self.sanity_check(keyfile, id)
+            return self.sanity_check(os.path.abspath(keyfile), id)
         keys_dir = get_keys_dir()
         for name in os.listdir(keys_dir):
             filename = os.path.join(keys_dir, name)
@@ -703,7 +708,7 @@ class KeyfileKey(ID_HMAC_SHA_256, KeyfileKeyBase):
     def get_new_target(self, args):
         keyfile = os.environ.get('BORG_KEY_FILE')
         if keyfile:
-            return keyfile
+            return os.path.abspath(keyfile)
         filename = args.location.to_key_filename()
         path = filename
         i = 1
